
import numpy as np

import matplotlib.pyplot as plt
import shapefile
from shapely.geometry import shape, Point

import cartopy.io.shapereader as shpreader
import shapely.geometry as sgeom
from shapely.ops import unary_union
from shapely.prepared import prep

def remove_continent_ocean(var_in,latitude,longitude,remove='continent'):
    '''
    DESCRIPTION
    Remove continent or ocean for input data, the remove layer aim is replaced 
    with NaN values.

    PARAMETERS
    :param var_in: float
    :param latitude: float
    :param longitude: float
    :param remove: string

    This function needs a principal data input that is var_in (3d numpy-array), 
    latitude (1d numpy-array), longitude (1d numpy-array) and remove (string).
    Where remove is only a number to represent the extraction objective.
    
    remove options:
    * for remove continent: remove = 'continent'
    * for remove ocean    : remove = 'ocean'

    EXAMPLE
    Read and define hgt, latitude and longitude:
    >>> data_dir = '/home/user/Data/Hgt_1000hPa_Dec49_Feb20.nc'
    >>> data = ca.read_netcdf(data_dir,1)
    >>> hgt = data['hgt'][:,0,:,:]
    >>> lat = data['latitude']
    >>> lon = data['longitude']

    Extract continent
    >>> hgt_without_continent = ca.remove_continent_ocean(hgt,lat,lon,'continent')
    
    Extract ocean
    >>> hgt_without_ocean = ca.remove_continent_ocean(hgt,lat,lon,'ocean')
    '''
    
    land_shp_fname = shpreader.natural_earth(resolution='50m',category='physical', name='land')

    land_geom = unary_union(list(shpreader.Reader(land_shp_fname).geometries()))
    land = prep(land_geom)
    
    def is_land(x, y):
        return land.contains(sgeom.Point(x, y))

    len_in = len(np.shape(var_in))
    
    lat_mi = min(latitude)
    lat_ma = max(latitude)
    lon_mi = min(longitude)
    lon_ma = max(longitude)
    
    len_latitude = len(np.shape(latitude))
    if len_latitude != 1:
        print('ERROR: latitude doesn\'t have 1 dimension')
        print('Actual latitude Dimension: ',len_latitude)
        print('')
        var_out = 0
        return var_out
    
    len_longitude = len(np.shape(longitude))
    if len_longitude != 1:
        print('ERROR: longitude doesn\'t have 1 dimension')
        print('Actual longitude Dimension: ',len_longitude)
        print('')
        var_out = 0
        return var_out
    
    var_out = np.copy(var_in)
    
    if len_in == 2:
        
        if remove == 'continent' or remove == 'Continent' or remove == '1':
            for i in longitude:
                for j in latitude:
                    if land.contains(sgeom.Point(i,j)) == True:
                        a, = np.where(latitude == j)
                        b, = np.where(longitude == i)
                        var_out[a,b] = np.nan
            return var_out
        
        if remove == 'ocean' or remove == 'Ocean' or remove == '2':
            for i in longitude:
                for j in latitude:
                    if land.contains(sgeom.Point(i,j)) == False:
                        a, = np.where(latitude == j)
                        b, = np.where(longitude == i)
                        var_out[a,b] = np.nan
            return var_out
        
        else:
            print('')
            print('ERROR: Option',remove,'doesn\'t exist, choose one:')
            print('1: continent')
            print('2: ocean')
            var_out = 0
            return var_out
        
    elif len_in == 3:
        
        if remove == 'continent' or remove == 'Continent' or remove == '1':
            for i in longitude:
                for j in latitude:
                    if land.contains(sgeom.Point(i,j)) == True:
                        a, = np.where(latitude == j)
                        b, = np.where(longitude == i)
                        var_out[:,a,b] = np.nan
            return var_out
        
        if remove == 'ocean' or remove == 'Ocean' or remove == '2':
            for i in longitude:
                for j in latitude:
                    if land.contains(sgeom.Point(i,j)) == False:
                        a, = np.where(latitude == j)
                        b, = np.where(longitude == i)
                        var_out[:,a,b] = np.nan
            return var_out
    
        else:
            print('')
            print('ERROR: Option',remove,'doesn\'t exist, choose one:')
            print('1: continent')
            print('2: ocean')
            var_out = 0
            return var_out
                    
    else:
        print('ERROR: the variable doesn\'t have 2 or 3 dimensions, please convert to (lat,lon) or (time,lat,lon)')
        print('Actual variable Dimension: ',len_in)
        print('')
        var_out = 0
        
        return var_out

def extract_shapefile(shapefile_dir,data_in,lat_in,lon_in,shp_id=0):
    '''
    DESCRIPTION
    Extract data into a shapefile and input data, where the output is only 
    the intersection.

    PARAMETERS
    :param shapefile_dir: string
    :param data_in: float
    :param lat_in: float
    :param lon_in: float
    :param shp_id: int

    This function needs a shapefile directory define with a variable shapefile_dir
    (string), the variable for extract define as data_in (3d numpy-array), 
    latitude (1d numpy-array), longitude (1d numpy-array) and one identifier
    for specific layer number into the shapefile, shp_id (integer).

    shp_id options:
    this parameter is the number of layers into shapefile file, for default 
    the shp_id is equal zero.

    EXAMPLE
    Read a define hgt, latitude and longitude:
    >>> data_dir = '/home/user/Data/Hgt_1000hPa_Dec49_Feb20.nc'
    >>> data_in = ca.read_netcdf(data_dir,1)
    >>> hgt = data['hgt'][:,0,:,:]
    >>> lat = data['latitude']
    >>> lon = data['longitude']

    Extract shapefile for hgt:
    >>> shapefile_dir = '/shapes/continent.shp'
    >>> shp_id = 0
    >>> hgt_intersection = ca.extract_shapefile(shapefile_dir,hgt,lat,lon,shp_id)
    '''
    
    r = shapefile.Reader(shapefile_dir)

    shapes = r.shapes()
    print('Shapes ID len: ',len(shapes))
    polygon = shape(shapes[shp_id]) 
    
    data_out = np.copy(data_in)
    
    if len(data_in.shape) == 1:
        for i in range(len(lat_in)):
            a = Point(lon_in[i],lat_in[i])

            if polygon.contains(a) == False:
                data_out[i] = np.nan
            
        return data_out
    
    if len(data_in.shape) == 2 or len(data_in.shape) == 3:
        for j in lat_in:
            for i in lon_in:
                a = Point(i,j)
                if polygon.contains(a) == False:
                    temp1, = np.where(lon_in == i)
                    temp2, = np.where(lat_in == j)

                    if len(data_in.shape) == 2:
                        data_out[temp2,temp1] = np.nan
                    elif len(data_in.shape) == 3:
                        data_out[:,temp2,temp1] = np.nan

        return data_out


import numpy as np
import netCDF4
from netCDF4 import Dataset
from netCDF4 import date2num

import calendar
import datetime
from datetime import timedelta
from dateutil.rrule import rrule, YEARLY, MONTHLY, WEEKLY, DAILY, HOURLY, MINUTELY

def read_netcdf(filename,show=1):
    """
    DESCRIPTION
    Simple function to read netcdf files.  
    
    PARAMETERS
    :param namefile: string 
    :param id: integer

    The namefile is the name of the netcdf file and id is the descriptor 
    number to show the different levels of details of the variables inside 
    the netcdf.

    id levels:
    * ca.read_netcdf(data_dir,3)
    Shows all the information contained about the variables

    * ca.read_netcdf(data_dir,2)
    Shows name of variables and its shapes

    * ca.read_netcdf(data_dir,1)
    Shows only the number of variables

    * ca.read_netcdf(data_dir,0)
    Shows nothing
    
    EXAMPLE
    Read and define hgt, latitude and longitude:
    >>> data_dir = '/home/user/Data/Hgt_1000hPa_Dec49_Feb20.nc'
    >>> data = ca.read_netcdf(data_dir,1)
    >>> hgt = data['hgt'][:,0,:,:]
    >>> lat = data['latitude']
    >>> lon = data['longitude']
    """

    data = Dataset(filename,mode = 'r')
    
    var_total = data.variables
    
    var_list = {}
    nn = 0
    for key, value in var_total.items():
        var_list[nn] = key
        nn = nn + 1
        
    var_len = len(var_list)
    
    for i in range(var_len):
        globals()[var_list[i]] = data.variables[var_list[i]][:]
    
    dict_out = dict()
    
    for i in range(var_len):
        dict_out[var_list[i]] = globals()[var_list[i]]

    if show == 0:
        pass

    elif show == 1:
        print('Number of variables: ',var_len)

    elif show == 2:
        for i in range(var_len):
            print(var_list[i],':',np.shape(globals()[var_list[i]]))

    elif show == 3:
        print(data.variables)

    else:
        print(f'Error: Id value {show} doesn\'t exits')
        print('Try with Id: 0, 1, 2 or 3')
        return None
    
    if 'time' in var_total:
        time_0 = data.variables['time'][:]
        time_1 = netCDF4.num2date(time_0[:],data.variables['time'].units)
        dict_out['time'] = time_1
    elif 't' in var_total:
        time_0 = data.variables['t'][:]
        time_1 = netCDF4.num2date(time_0[:],data.variables['t'].units)
        dict_out['t'] = time_1
    
    return dict_out

def create_netcdf(info,data,latitude,longitude):
    """
    DESCRIPTION
    Function for export data numpy array to netcdf format.

    PARAMETERS
    :param latitude: float 
    :param longitude: float
    :param var_input: float

    This function uses four inputs. First an info dictionary called info that
    contains the principal information about the netcdf file what will be 
    created, the data input matrix (3d numpy-array), latitude (1d numpy-array) 
    and longitude (1d numpy-array).

    The info is define as:
    info = {'file': filename_out, # filename for netcdf file
            'title': description, # netcdf title or little descriptionn
            'year_start':y_s,'month_start':m_s,'day_start':d_s,
            'hour_start':h_s,'minute_start':m_s, # time start 
            'year_end':y_e,'month_end':m_e,'day_end':d_e,
            'hour_end':h_e,'minute_end':m_e, # time end
            'time_frequency': frequency, # Monthly, Daily, Hourly, Minutely
            'time_interval':time_int, # time step interval
            'var_name': var_name, # variable short name
            'var_units': var_units} # variable units

    Where:
    :param file: string
    :param title: string
    :param year_start month_start day_start hour_start minute_start: integer
    :param year_end month_end day_end hour_end minute_end: integer
    :param time_frequency: string
    :param time_interval: integer
    :param var_name: sring
    :param var_units: string

    time_frequency options:
    This parameter define the time resolution of data
    * 'time_frequency': Monthly
    * 'time_frequency': Daily
    * 'time_frequency': Hourly
    * 'time_frequency': Minutely

    EXAMPLE
    Read and define hgt, latitude and longitude:
    >>> data_dir = '/home/user/Data/Hgt_1000hPa_Dec49_Feb20.nc'
    >>> data = ca.read_netcdf(data_dir,1)
    >>> hgt = data['hgt'][:,0,:,:]
    >>> lat = data['latitude']
    >>> lon = data['longitude']

    Define info for the new netcdf about hgt:
    >>> info = {
        'file': '/home/user/Data/Hgt_1000hPa_Dec49_Feb20_new.nc',
        'title': 'Monthly Geopotential Height 1000 hPa', 
        'year_start':1949, 'month_start':12, 'day_start':1, 
        'hour_start':0, 'minute_start':0, 
        'year_end':2020, 'month_end':2, 'day_end':1, 
        'hour_end':23, 'minute_end':59, 
        'time_frequency': 'Monthly', 
        'time_interval':1,
        'var_name': 'hgt',
        'var_units': 'hPa'} 

    Create netcdf file:
    >>> ca.create_netcdf(info,hgt,lat,lon)
    """
    
    if info['year_start'] > info['year_end']:
        print('ERROR: Year Start cannot be greater than Year End')
        dates = 0
        return dates

    str_dt = datetime.datetime(info['year_start'],info['month_start'],info['day_start'],info['hour_start'],info['minute_start'])
    end_dt = datetime.datetime(info['year_end'],info['month_end'],info['day_end'],info['hour_end'],info['minute_end'])
    
    if info['time_frequency'] == 'Monthly' or info['time_frequency'] == 'monthly':
        dates = [dt for dt in rrule(MONTHLY, interval=info['time_interval'], dtstart=str_dt, until=end_dt)]
        print('Time Start:',dates[0])
        print('Time End:',dates[-1])
        print('Time Frequency:',info['time_frequency'])
        print('Time Lenght:',len(dates))
    
    elif info['time_frequency'] == 'Weekly' or info['time_frequency'] == 'weekly':
        dates = [dt for dt in rrule(WEEKLY, interval=info['time_interval'], dtstart=str_dt, until=end_dt)]
        print('Time Start:',dates[0])
        print('Time End:',dates[-1])
        print('Time Frequency:',info['time_frequency'])
        print('Time Lenght:',len(dates))

    elif info['time_frequency'] == 'Daily' or info['time_frequency'] == 'daily':
        dates = [dt for dt in rrule(DAILY, interval=info['time_interval'], dtstart=str_dt, until=end_dt)]
        print('Time Start:',dates[0])
        print('Time End:',dates[-1])
        print('Time Frequency:',info['time_frequency'])
        print('Time Lenght:',len(dates))
           
    elif info['time_frequency'] == 'Hourly' or info['time_frequency'] == 'hourly':
        dates = [dt for dt in rrule(HOURLY, interval=info['time_interval'], dtstart=str_dt, until=end_dt)]
        print('Time Start:',dates[0])
        print('Time End:',dates[-1])
        print('Time Frequency:',info['time_frequency'])
        print('Time Lenght:',len(dates))
    
    elif info['time_frequency'] == 'Minutely' or info['time_frequency'] == 'minutely':
        dates = [dt for dt in rrule(MINUTELY, interval=info['time_interval'], dtstart=str_dt, until=end_dt)]
        print('Time Start:',dates[0])
        print('Time End:',dates[-1])
        print('Time Frequency:',info['time_frequency'])
        print('Time Lenght:',len(dates))
        
    else:
        print('ERROR: time_frequency \"' + info['time_frequency'] +'\" not defined')
        dates = 'time_frequency should be: Monthly, Weekly, Daily, Hourly or Minutely'
        return dates
    
    tiempo = np.zeros([len(dates)])

    tiempo = date2num(dates, units='days since '+str(info['year_start'])+'-'+str(info['month_start'])+'-'+str(info['day_start']))

    info_a = { 'comment': info['time_frequency']+' '+info['title']+' from '+str(info['year_start'])+'-'+str(info['month_start'])+'-'+str(info['day_start'])+' to december '+str(info['year_end'])+'-'+str(info['month_end'])+'-'+str(info['day_end']),
           'time_units': 'days since '+str(info['year_start'])+'-'+str(info['month_start'])+'-'+str(info['day_start'])} # don't edit
    
    nc_title = info['title']
    nc_comment = info_a['comment']

    nc_time_units = info_a['time_units']

    nc_var_name = info['var_name']
    nc_var_units = info['var_units']
    
    ncfile = netCDF4.Dataset(info['file'],mode='w',format='NETCDF4_CLASSIC')
    
    ncfile.title = nc_title
    ncfile.anything = nc_comment
    
    lat_dim = len(latitude)
    lon_dim = len(longitude)
    time_dim = len(tiempo)
    
    lat_dim = ncfile.createDimension('lat',lat_dim)
    lon_dim = ncfile.createDimension('lon',lon_dim)
    time_dim = ncfile.createDimension('time',None)
    
    lat = ncfile.createVariable('lat',np.float32,('lat',))
    lat.units = 'degrees_north'
    lat.long_name = 'latitude'

    lon = ncfile.createVariable('lon',np.float32,('lon',))
    lon.units = 'degrees_east'
    lon.long_name = 'longitude'

    time = ncfile.createVariable('time',np.float64,('time',))
    time.units = nc_time_units
    time.long_name = 'time'
    
    var = ncfile.createVariable(info['var_name'],np.float64,('time','lat','lon'))
    var.units = nc_var_units
    var.standard_name = nc_var_name
    
    lat[:] = latitude
    lon[:] = longitude
    
    var[:,:,:] = data
    
    time[:] = tiempo
    
    print()
    print('File created in: ',info['file'])
    print('File title: ',info['title'])
    print('var title: ',info['var_name'])
    print('var units: ',info['var_units'])
    
    ncfile.close()



import numpy as np
import matplotlib.colors
from matplotlib import pyplot as plt
import calendar

def redefinir_area(data_in,lat_in,lon_in,lat_min,lat_max,lon_min,lon_max):
    '''
    DESCRIPTION
    Funntion to extract a specific region 

    PARAMETERS
    :param data_in: float
    :param lat_in: float
    :param lon_in: float
    :param lat_min: float
    :param lat_max: float
    :param lon_min: float
    :param lon_max: float

    EXAMPLE
    Read and define hgt, latitude and longitude:
    >>> data_dir = '/home/user/Data/Hgt_1000hPa_Dec49_Feb20.nc'
    >>> data = ca.read_netcdf(data_dir,1)
    >>> hgt = data['hgt'][:,0,:,:]
    >>> lat = data['latitude']
    >>> lon = data['longitude']

    Define limits for region 
    >>> lat_min = -17; lat_max = 17
    >>> lon_min = -50; lon_max = 15

    Extract area
    >>> hgt_cut, lat_cut, lon_cut = redefinir_area(hgt,lat,lon,lat_min,lat_max,lon_min,lon_max)

    '''
    
    y_bounds = np.where((lat_in>=lat_min)&(lat_in<=lat_max))[0]
    lat_tmp = lat_in[y_bounds]
    
    x_bounds = np.where((lon_in>=lon_min)&(lon_in<=lon_max))[0]
    lon_tmp = lon_in[x_bounds]

    data_out = data_in[:,y_bounds[0]:y_bounds[-1]+1,x_bounds[0]:x_bounds[-1]+1]
    
    return data_out, lat_tmp, lon_tmp

def DiasDoAno(ano,mes):
    '''
    DESCRIPTION
    This Function shows start and end day of year.

    PARAMETERS
    :param ano: integer
    :param mes: string

    EXAMPLE
    >>> DiasDoAno(2015,feb)
    >>> DiasDoAno(2015,Feb)
    >>> DiasDoAno(2015,February)
    '''
    
    if calendar.isleap(ano) == True:

        if mes == 'jan' or mes == 'Jan' or mes == 'January': 
            day_start=1;day_end=31
        elif mes == 'feb' or mes == 'Feb' or mes == 'February':
            day_start=32;day_end=60
        elif mes == 'mar' or mes == 'Mar' or mes == 'March':
            day_start=61;day_end=91
        elif mes == 'apr' or mes == 'Apr' or mes == 'April':
            day_start=92;day_end=121
        elif mes == 'may' or mes == 'May' or mes == 'May':
            day_start=122;day_end=152
        elif mes == 'jun' or mes == 'Jun' or mes == 'June':
            day_start=153;day_end=182
        elif mes == 'jul' or mes == 'Jul' or mes == 'July':
            day_start=183;day_end=213
        elif mes == 'ago' or mes == 'Ago' or mes == 'August':
            day_start=214;day_end=244
        elif mes == 'sep' or mes == 'Sep' or mes == 'September':
            day_start=245;day_end=274
        elif mes == 'oct' or mes == 'Oct' or mes == 'October':
            day_start=275;day_end=305
        elif mes == 'nov' or mes == 'Nov' or mes == 'November':
            day_start=306;day_end=335
        elif mes == 'dec' or mes == 'Dec' or mes == 'December':
            day_start=336;day_end=366
        else:
            print('ERROR: Month name')
            day_start=0;day_end=0
            return day_start,day_end

    else:

        if mes == 'jan' or mes == 'Jan' or mes == 'January':
            day_start=1;day_end=31
        elif mes == 'feb' or mes == 'Feb' or mes == 'February':
            day_start=32;day_end=59
        elif mes == 'mar' or mes == 'Mar' or mes == 'March':
            day_start=60;day_end=90
        elif mes == 'apr' or mes == 'Apr' or mes == 'April':
            day_start=91;day_end=120
        elif mes == 'may' or mes == 'May' or mes == 'May':
            day_start=121;day_end=151
        elif mes == 'jun' or mes == 'Jun' or mes == 'June':
            day_start=152;day_end=181
        elif mes == 'jul' or mes == 'Jul' or mes == 'July':
            day_start=182;day_end=212
        elif mes == 'ago' or mes == 'Ago' or mes == 'August':
            day_start=213;day_end=243
        elif mes == 'sep' or mes == 'Sep' or mes == 'September':
            day_start=244;day_end=273
        elif mes == 'oct' or mes == 'Oct' or mes == 'October':
            day_start=274;day_end=304
        elif mes == 'nov' or mes == 'Nov' or mes == 'November':
            day_start=305;day_end=334
        elif mes == 'dec' or mes == 'Dec' or mes == 'December':
            day_start=335;day_end=365
        else:
            print('ERROR: Month name')
            day_start=0;day_end=0
            return day_start,day_end
                
    return day_start,day_end

def colorbar_middle_white(cmap,position='middle',n=35,x=0.5):
    """
    DESCRIPTION
    Function to add middle white from another colorbar 

    PARAMETERS
    :param cmap: colormap
    :param position: string or integer
    :param n: integer
    :param x: float

    from a colorbar definas as cmap, n as the number of nivels and the increment, 
    this is a simple function to modify a specific colorbar.

    The parameter position input could be defined as:
    1. Starting with white: position = 'left' or position = -1
    2. midlle with white: position = 'middle' or position = 0
    3. ending with white: position = 'right' or position = 1   

    EXAMPLE
    This example uses a red to blue colorbar 
    >>> cmap = plt.cm.Spectral_r
    Colorbar with white in the left
    >>> cmap_sst = colorbar_middle_white(cmap,-1)
    Colorbar with white in the middle
    >>> cmap_sst = colorbar_middle_white(cmap,0)
    Colorbar with white in the right
    >>> cmap_sst = colorbar_middle_white(cmap,1)
    """

    lower = cmap(np.linspace(0, x, n))
    white = np.ones((80-2*n,4))
    upper = cmap(np.linspace(1-x, 1, n))

    if position == 'left' or position == -1:
        colors = np.vstack((white, lower, upper))
    elif position == 'middle' or position == 0:
        colors = np.vstack((lower, white, upper))
    elif position == 'right' or position == 1:
        colors = np.vstack((lower, upper, white))
    else:
        print(f'ERROR: position {position} is not defined')
        print('Try to: left, middle, right or -1, 0, 1')
        tmap = None
        return tmap

    tmap = matplotlib.colors.LinearSegmentedColormap.from_list('map_white', colors)

    return tmap





import numpy as np

def data_dictionary(var_in):
    """
    DESCRIPTION
    Convert monthly data input in pandas DataFrame dictionary. 
    
    PARAMETERS
    :param var_input: float

    This function only needs the data input (3d numpy-array), where this data is 
    define as [time, latitude, longitude]. 

    The DataFrame output it composed for three levels:
    1. data (monthly original data input)
    2. clim (monthly climatologies)
    3. anom (monthly anomalies)

    EXAMPLE
    Read and define hgt:
    >>> data_dir = '/home/user/Data/Hgt_1000hPa_Dec49_Feb20.nc'
    >>> data = ca.read_netcdf(data_dir,1)
    >>> hgt = data['hgt'][:,0,:,:]

    Create a dictionary
    >>> hgt_dictionary = ca.data_dictionary(hgt)

    Obtain the climatology and anomalies for each july
    >>> jul_anom = hgt_dictionary['jul']['anom']
    >>> jul_clim = hgt_dictionary['jul']['clim']
    """

    len_in = len(np.shape(var_in))

    if len_in == 3:
        
        time,lat,lon = np.shape(var_in)
    
        lat = int(lat)
        lon = int(lon)
        anos = int(time/12)      
        
        var_name = {"jan": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},
                    "feb": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},
                    "mar": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},
                    "apr": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},
                    "may": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},
                    "jun": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},
                    "jul": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},
                    "ago": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},
                    "sep": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},
                    "oct": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},
                    "nov": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},
                    "dec": {'data':np.zeros([anos,lat,lon]),
                            'clim':np.zeros([1,lat,lon]),
                            'anom':np.zeros([anos,lat,lon])},}
        
        print('')
        print('Keys level 1: ',var_name.keys())
        print('keys level 2: ',var_name['jan'].keys())
        print('')
        print('Numbers of years: ',anos)
        print('Numbers of months: ',time)

        for i in range(0,anos):
            for j, key1 in enumerate(var_name.keys()):
                var_name[key1]['data'][i,:,:] = var_in[12*i+j,:,:]

        for i, key1 in enumerate(var_name.keys()):
            var_name[key1]['clim'][0,:,:] = (sum((var_name[key1]['data'][i,:,:]) for i in range(0,anos)))/anos

        for i in range(anos):
            for j, key1 in enumerate(var_name.keys()):
                var_name[key1]['anom'][i, :, :] = var_name[key1]['data'][i, :, :] - var_name[key1]['clim']
                
        return var_name
        
    else:
        print('ERROR: the variable doesn\'t have 3 dimensions, please convert to (time,lat,lon)')
        print('Actual variable Dimension: ',len_in)
        print('')
        var_name = 0
        
        return var_name

def climatology(var_in):
    """
    DESCRIPTION
    Create a for monthly climatology.

    PARAMETERS
    :param var_in: DataFrame

    This function uses the output to ca.data_dictionary.

    EXAMPLE
    Read and define hgt:
    >>> data_dir = '/home/user/Data/Hgt_1000hPa_Dec49_Feb20.nc'
    >>> data = ca.read_netcdf(data_dir,1)
    >>> hgt = data['hgt'][:,0,:,:]

    Create a hgt dictionary
    >>> hgt_dictionary = ca.data_dictionary(hgt)

    Obtain the monthly climatologies matrix
    >>> hgt_climatology = ca.climatology(hgt_dictionary)
    """

    num_anos,len_lat,len_lon = np.shape(var_in['jan']['data'])
    var_out = np.zeros([12,len_lat,len_lon])
    
    var_out[0,:,:] = var_in['jan']['clim']
    var_out[1,:,:] = var_in['feb']['clim']
    var_out[2,:,:] = var_in['mar']['clim']
    var_out[3,:,:] = var_in['apr']['clim']
    var_out[4,:,:] = var_in['may']['clim']
    var_out[5,:,:] = var_in['jun']['clim']
    var_out[6,:,:] = var_in['jul']['clim']
    var_out[7,:,:] = var_in['ago']['clim']
    var_out[8,:,:] = var_in['sep']['clim']
    var_out[9,:,:] = var_in['oct']['clim']
    var_out[10,:,:] = var_in['nov']['clim']
    var_out[11,:,:] = var_in['dec']['clim']
        
    return var_out

def anomalies(var_in):
    """
    DESCRIPTION
    Create a for monthly anomaly.

    PARAMETERS
    :param var_in: DataFrame

    This function uses the output to ca.data_dictionary.

    EXAMPLE
    Read and define hgt:
    >>> data_dir = '/home/user/Data/Hgt_1000hPa_Dec49_Feb20.nc'
    >>> data = ca.read_netcdf(data_dir,1)
    >>> hgt = data['hgt'][:,0,:,:]

    Create a hgt dictionary
    >>> hgt_dictionary = ca.data_dictionary(hgt)

    Obtain the monthly anomalies matrix
    >>> hgt_climatology = ca.anomalies(hgt_dictionary)
    """

    num_anos,len_lat,len_lon = np.shape(var_in['jan']['data'])
    var_out = np.zeros([12*num_anos,len_lat,len_lon])
    
    for i in range(num_anos):
        var_out[12*i,:,:] = var_in['jan']['anom'][i,:,:]
        var_out[12*i+1,:,:] = var_in['feb']['anom'][i,:,:]
        var_out[12*i+2,:,:] = var_in['mar']['anom'][i,:,:]
        var_out[12*i+3,:,:] = var_in['apr']['anom'][i,:,:]
        var_out[12*i+4,:,:] = var_in['may']['anom'][i,:,:]
        var_out[12*i+5,:,:] = var_in['jun']['anom'][i,:,:]
        var_out[12*i+6,:,:] = var_in['jul']['anom'][i,:,:]
        var_out[12*i+7,:,:] = var_in['ago']['anom'][i,:,:]
        var_out[12*i+8,:,:] = var_in['sep']['anom'][i,:,:]
        var_out[12*i+9,:,:] = var_in['oct']['anom'][i,:,:]
        var_out[12*i+10,:,:] = var_in['nov']['anom'][i,:,:]
        var_out[12*i+11,:,:] = var_in['dec']['anom'][i,:,:]

    return var_out

def season(var_in,season=1):
    """
    DESCRIPTION
    This function separates the data into seasons and create two ouput arrays, 
    one of climatologies and another of anomalies.

    PARAMETERS
    :param var_in: DataFrame
    :param season: integer

    This function uses the output to ca.data_dictionary and one indicator for the 
    specific season define as season.

    season options:
    * for summer season = 1
    * for autumn season = 2
    * for winter season = 3
    * for spring season = 4

    EXAMPLE
    Read and define hgt:
    >>> data_dir = '/home/user/Data/Hgt_1000hPa_Dec49_Feb20.nc'
    >>> data = ca.read_netcdf(data_dir,1)
    >>> hgt = data['hgt'][:,0,:,:]
    
    Create a hgt dictionary
    >>> hgt_dictionary = ca.data_dictionary(hgt)

    Obtain the climatologies and anomalies for each season
    >>> clima_summer, anom_summer = ca.season(hgt_dictionary,1)
    >>> clima_autumn, anom_autumn = ca.season(hgt_dictionary,2)
    >>> clima_winter, anom_winter = ca.season(hgt_dictionary,3)
    >>> clima_spring, anom_spring = ca.season(hgt_dictionary,4)
    """

    anos,len_lat,len_lon = np.shape(var_in['jan']['data'])
    
    if season == 1:

        print('SUMMER Climatology')

        clima = 0
        clima = (var_in['dec']['clim'][0,:,:] + var_in['jan']['clim'][0,:,:] + var_in['feb']['clim'][0,:,:])/3
        
        anom = np.zeros([(anos)-1,len_lat,len_lon])
        for i in range(anos-1):
            anom[i,:,:]=(var_in['dec']['anom'][i,:,:]+var_in['jan']['anom'][i+1,:,:]+var_in['feb']['anom'][i+1,:,:])/3
         
        print('')
        print('Climatology: ',np.shape(clima))
        print('Anomaly: ',np.shape(anom))
        print('')

        return clima,anom

    elif season == 2:

        print('AUTUMN climatology')

        clima = 0
        clima = (var_in['mar']['clim'][0,:,:] + var_in['apr']['clim'][0,:,:] + var_in['may']['clim'][0,:,:])/3
        
        anom = np.zeros([(anos)-1,len_lat,len_lon])
        for i in range(anos-1):
            anom[i,:,:]=(var_in['mar']['anom'][i,:,:]+var_in['apr']['anom'][i+1,:,:]+var_in['may']['anom'][i+1,:,:])/3
         
        print('')
        print('Climatology: ',np.shape(clima))
        print('Anomaly: ',np.shape(anom))

        return clima,anom

    elif season == 3:

        print('WINTER Climatology')

        clima = 0
        clima = (var_in['jun']['clim'][0,:,:] + var_in['jul']['clim'][0,:,:] + var_in['ago']['clim'][0,:,:])/3
        
        anom = np.zeros([(anos)-1,len_lat,len_lon])
        for i in range(anos-1):
            anom[i,:,:]=(var_in['jun']['anom'][i,:,:]+var_in['jul']['anom'][i+1,:,:]+var_in['ago']['anom'][i+1,:,:])/3
         
        print('')
        print('Climatology: ',np.shape(clima))
        print('Anomaly: ',np.shape(anom))

        return clima,anom

    elif season == 4:

        print('SPRING Climatology')

        clima = 0
        clima = (var_in['sep']['clim'][0,:,:] + var_in['oct']['clim'][0,:,:] + var_in['nov']['clim'][0,:,:])/3
        
        anom = np.zeros([(anos)-1,len_lat,len_lon])
        for i in range(anos-1):
            anom[i,:,:]=(var_in['sep']['anom'][i,:,:]+var_in['oct']['anom'][i+1,:,:]+var_in['nov']['anom'][i+1,:,:])/3
         
        print('')
        print('Climatology: ',np.shape(clima))
        print('Anomaly: ',np.shape(anom))

        return clima,anom

    else:
        print('')
        print('ERROR: Season doesn\'t exist, choose one:')
        print('1: Summer')
        print('2: Autumn')
        print('3: winter')
        print('4: Spring')
        anom = 0
        clima = 0  

        return anom,clima


from setuptools import setup, find_packages
import os
import pathlib
import sys

HERE = pathlib.Path(__file__).parent.resolve()
LONG_DESCRIPTION = (HERE / 'README.md').read_text(encoding='utf-8')

requires = [
    'numpy==1.21.2',
    'pandas==1.3.4',
]

setup(
    name='clima_anom',
    version='0.7.0',
    description='Obtain the climatology and anomalies only for monthly data.',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url='https://github.com/mhacarthur/clima-anom',
    author='Cesar Arturo Sanchez Pena',
    author_email='arturo66cta@gmail.com',
    license='MIT',
    # packages=find_packages(where=HERE),
    packages=find_packages(where='.'+os.sep),
    python_requires='>=3.5, <=3.10.1',
    install_requires=requires,
    keywords=['climatology','anomalies','monthly','correlation','netcdf','MAE','RMSE','BIAS','Create netcdf','remove continent','remove ocean','shapefile remove'],
    zip_safe=False,
)




import numpy as np

def correlation(data_1,data_2):
    '''
    DESCRIPTION
    Computes the matrix correlation into two time series

    PARAMETERS
    :param data_1: float
    :param data_2: float

    This funtion use two time series (1d numpy-array)

    EXAMPLE
    Read and define hgt:
    >>> data_dir = '/home/user/Data/Hgt_1000hPa_Dec49_Feb20.nc'
    >>> data = ca.read_netcdf(data_dir,1)
    >>> hgt = data['hgt'][:,0,:,:]

    Define two series inside hgt
    >>> serie_1 = hgt[:,10,12]
    >>> serie_2 = hgt[:,5,8]

    Obtain the correlation into series
    >>> serie_correlation = ca.correlation(serie_1,serie_2)
    '''
    
    anos,len_lat,len_lon = np.shape(data_1)
    
    corr = np.zeros([len_lat,len_lon])
    
    for i in range(len_lat):
        for j in range(len_lon):
            
            nan_bnd = (np.argwhere(~np.isnan(data_1[:,i,j])))[:,0]
            pearson = np.corrcoef(data_1[:,i,j][nan_bnd],data_2[:,i,j][nan_bnd])
            corr_temp = pearson[0,1]
            corr[i,j] = corr_temp
            
    return corr

def MAE(data1,data2):
    '''
    DESCRIPTION
    Obtain the MAE into two 3d arrays

    PARAMETERS
    :param data1: float
    :param data2: float

    Where data1 and data2 are 3d numpy array
    '''
    
    if data1.ndim != 3:
        print('')
        print('ERROR: data1 does not have 3 dimentions')
        print(np.shape(data1))
        MAE = 0
        return MAE
        
    if data2.ndim != 3:
        print('')
        print('ERROR: data2 does not have 3 dimentions')
        print(np.shape(data2))
        MAE = 0
        return MAE
    
    t1,n1,m1 = np.shape(data1)
    t2,n2,m2 = np.shape(data2)
    
    if n1 != n2:
        print('')
        print('ERROR: Matrices do not have the same latitude dimension')
        MAE = 0
        return MAE
    elif m1 != m2:
        print('')
        print('ERROR: Matrices do not have the same longitude dimension')
        MAE = 0
        return MAE
    elif t1 != t2:
        print('')
        print('ERROR: Matrices do not have the same time dimension')
        MAE = 0
        return MAE
    
    DIFF = abs(data1 - data2)
    
    MAE = np.zeros([n1,m1])
    
    for i in range(n1):
        for j in range(m1):
            MAE[i,j] = np.sum(DIFF[:,i,j])/t1
        
    return MAE

def RMSE(data1,data2):
    '''
    DESCRIPTION
    Obtain the RMSE into two 3d arrays

    PARAMETERS
    :param data1: float
    :param data2: float

    Where data1 and data2 are 3d numpy array
    '''
    
    if data1.ndim != 3:
        print('')
        print('ERROR: Data1 does not have 3 dimensions')
        RMSE = 0
        return RMSE
        
    if data2.ndim != 3:
        print('')
        print('ERROR: Data2 does not have 3 dimensions')
        RMSE = 0
        return RMSE
    
    t1,n1,m1 = np.shape(data1)
    t2,n2,m2 = np.shape(data2)
    
    if n1 != n2:
        print('')
        print('ERROR: Matrices do not have the same latitude dimension')
        RMSE = 0
        return RMSE
    elif m1 != m2:
        print('')
        print('ERROR: Matrices do not have the same longitude dimension')
        RMSE = 0
        return RMSE
    elif t1 != t2:
        print('')
        print('ERROR: Matrices do not have the same time dimension')
        RMSE = 0
        return RMSE
    
    DIFF = (data1 - data2)
    
    RMSE = np.zeros([n1,m1])
    
    for i in range(n1):
        for j in range(m1):
            RMSE[i,j] = np.sqrt(np.sum(DIFF[:,i,j]*DIFF[:,i,j])/t1)
        
    return RMSE 

def BIAS(data1,data2):
    '''
    DESCRIPTION
    Obtain the BIAS into two 3d arrays

    PARAMETERS
    :param data1: float
    :param data2: float

    Where data1 and data2 are 3d numpy array
    '''
    
    if data1.ndim != 3:
        print('')
        print('ERROR: Data1 does not have 3 dimensions')
        BIAS = 0
        return BIAS
        
    if data2.ndim != 3:
        print('')
        print('ERROR: Data2 does not have 3 dimensions')
        BIAS = 0
        return BIAS
    
    t1,n1,m1 = np.shape(data1)
    t2,n2,m2 = np.shape(data2)
    
    if n1 != n2:
        print('')
        print('ERROR: Matrices do not have the same latitude dimension')
        BIAS = 0
        return BIAS

    elif m1 != m2:
        print('')
        print('ERROR: Matrices do not have the same longitude dimension')
        BIAS = 0
        return BIAS

    elif t1 != t2:
        print('')
        print('ERROR: Matrices do not have the same time dimension')
        BIAS = 0
        return BIAS
    
    BIAS = np.zeros([n1,m1])
    
    for i in range(n1):
        for j in range(m1):
            BIAS[i,j] = np.sum(data1[:,i,j] - data2[:,i,j])/t2
        
    return BIAS

def RBIAS(data1,data2):
    '''
    DESCRIPTION
    Obtain the RBIAS into two 3d arrays

    PARAMETERS
    :param data1: float
    :param data2: float

    Where data1 and data2 are 3d numpy array
    '''
    
    if data1.ndim != 3:
        print('')
        print('ERROR: Data1 does not have 3 dimensions')
        RBIAS = 0
        return RBIAS
        
    if data2.ndim != 3:
        print('')
        print('ERROR: Data2 does not have 3 dimensions')
        RBIAS = 0
        return RBIAS
    
    t1,n1,m1 = np.shape(data1)
    t2,n2,m2 = np.shape(data2)
    
    if n1 != n2:
        print('')
        print('ERROR: Matrices do not have the same latitude dimension')
        RBIAS = 0
        return RBIAS

    elif m1 != m2:
        print('')
        print('ERROR: Matrices do not have the same longitude dimension')
        RBIAS = 0
        return RBIAS

    elif t1 != t2:
        print('')
        print('ERROR: Matrices do not have the same time dimension')
        RBIAS = 0
        return RBIAS
    
    RBIAS = np.zeros([n1,m1])
    
    for i in range(n1):
        for j in range(m1):
            RBIAS[i,j] = ((np.sum(data1[:,i,j]-data2[:,i,j]))/(np.sum(data2[:,i,j])))*100
        
    return RBIAS

